@@ -762,7 +762,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
                                      needed (defaults to True)
         :param network: (optional) Network dict or name or ID to attach the
                         server to.  Mutually exclusive with the nics parameter.
-                        Can also be be a list of network names or IDs or
+                        Can also be a list of network names or IDs or
                         network dicts.
         :param boot_from_volume: Whether to boot from volume. 'boot_volume'
                                  implies True, but boot_from_volume=True with
@@ -790,6 +790,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
             raise TypeError(
                 "create_server() requires either 'image' or 'boot_volume'")
 
+        microversion = None
         server_json = {'server': kwargs}
 
         # TODO(mordred) Add support for description starting in 2.19
@@ -883,11 +884,14 @@ class ComputeCloudMixin(_normalize.Normalizer):
                         " may be given")
                 if fixed_ip:
                     net['fixed_ip'] = fixed_ip
-            # TODO(mordred) Add support for tag if server supports microversion
-            # 2.32-2.36 or >= 2.42
             for key in ('port', 'port-id'):
                 if key in nic:
                     net['port'] = nic.pop(key)
+            # A tag supported only in server microversion 2.32-2.36 or >= 2.42
+            # Bumping the version to 2.42 to support the 'tag' implementation
+            if 'tag' in nic:
+                microversion = utils.pick_microversion(self.compute, '2.42')
+                net['tag'] = nic.pop('tag')
             if nic:
                 raise exc.OpenStackCloudException(
                     "Additional unsupported keys given for server network"
@@ -927,7 +931,8 @@ class ComputeCloudMixin(_normalize.Normalizer):
             endpoint = '/os-volumes_boot'
         with _utils.shade_exceptions("Error in creating instance"):
             data = proxy._json_response(
-                self.compute.post(endpoint, json=server_json))
+                self.compute.post(endpoint, json=server_json,
+                                  microversion=microversion))
             server = self._get_and_munchify('server', data)
             admin_pass = server.get('adminPass') or kwargs.get('admin_pass')
             if not wait:
@@ -1628,6 +1633,16 @@ class ComputeCloudMixin(_normalize.Normalizer):
 
         :raises: OpenStackCloudException on operation error.
         """
+        if (
+            isinstance(name_or_id, six.string_types + (six.binary_type,))
+            and not name_or_id.isdigit()
+        ):
+            aggregate = self.get_aggregate(name_or_id)
+            if not aggregate:
+                self.log.debug(
+                    "Aggregate %s not found for deleting", name_or_id)
+                return False
+            name_or_id = aggregate.id
         try:
             self.compute.delete_aggregate(name_or_id, ignore_missing=False)
             return True
@@ -1798,7 +1813,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
         proj = self.get_project(name_or_id)
         if not proj:
             raise exc.OpenStackCloudException(
-                "project does not exist: {}".format(name=proj.id))
+                "project does not exist: {name}".format(name=proj.id))
 
         data = proxy._json_response(
             self.compute.get(
