@@ -17,6 +17,7 @@
 import argparse as argparse_mod
 import collections
 import copy
+import errno
 import json
 import os
 import re
@@ -101,7 +102,7 @@ def _fix_argv(argv):
     # Transform any _ characters in arg names to - so that we don't
     # have to throw billions of compat argparse arguments around all
     # over the place.
-    processed = collections.defaultdict(list)
+    processed = collections.defaultdict(set)
     for index in range(0, len(argv)):
         # If the value starts with '--' and has '-' or '_' in it, then
         # it's worth looking at it
@@ -113,7 +114,7 @@ def _fix_argv(argv):
                 split_args[0] = new
                 argv[index] = "=".join(split_args)
             # Save both for later so we can throw an error about dupes
-            processed[new].append(orig)
+            processed[new].add(orig)
     overlap = []
     for new, old in processed.items():
         if len(old) > 1:
@@ -140,7 +141,9 @@ class OpenStackConfig(object):
                  envvar_prefix=None, secure_files=None,
                  pw_func=None, session_constructor=None,
                  app_name=None, app_version=None,
-                 load_yaml_config=True, load_envvars=True):
+                 load_yaml_config=True, load_envvars=True,
+                 statsd_host=None, statsd_port=None,
+                 statsd_prefix=None, influxdb_config=None):
         self.log = _log.setup_logging('openstack.config')
         self._session_constructor = session_constructor
         self._app_name = app_name
@@ -252,6 +255,7 @@ class OpenStackConfig(object):
         self._cache_class = 'dogpile.cache.null'
         self._cache_arguments = {}
         self._cache_expirations = {}
+        self._influxdb_config = {}
         if 'cache' in self.cloud_config:
             cache_settings = _util.normalize_keys(self.cloud_config['cache'])
 
@@ -276,6 +280,44 @@ class OpenStackConfig(object):
             self._cache_expirations = cache_settings.get(
                 'expiration', self._cache_expirations)
 
+        if load_yaml_config:
+            metrics_config = self.cloud_config.get('metrics', {})
+            statsd_config = metrics_config.get('statsd', {})
+            statsd_host = statsd_host or statsd_config.get('host')
+            statsd_port = statsd_port or statsd_config.get('port')
+            statsd_prefix = statsd_prefix or statsd_config.get('prefix')
+
+            influxdb_cfg = metrics_config.get('influxdb', {})
+            # Parse InfluxDB configuration
+            if influxdb_config:
+                influxdb_cfg.update(influxdb_config)
+            if influxdb_cfg:
+                config = {}
+                if 'use_udp' in influxdb_cfg:
+                    use_udp = influxdb_cfg['use_udp']
+                    if isinstance(use_udp, str):
+                        use_udp = use_udp.lower() in ('true', 'yes', '1')
+                    elif not isinstance(use_udp, bool):
+                        use_udp = False
+                        self.log.warning('InfluxDB.use_udp value type is not '
+                                         'supported. Use one of '
+                                         '[true|false|yes|no|1|0]')
+                    config['use_udp'] = use_udp
+                for key in ['host', 'port', 'username', 'password', 'database',
+                            'measurement', 'timeout']:
+                    if key in influxdb_cfg:
+                        config[key] = influxdb_cfg[key]
+                self._influxdb_config = config
+
+        if load_envvars:
+            statsd_host = statsd_host or os.environ.get('STATSD_HOST')
+            statsd_port = statsd_port or os.environ.get('STATSD_PORT')
+            statsd_prefix = statsd_prefix or os.environ.get('STATSD_PREFIX')
+
+        self._statsd_host = statsd_host
+        self._statsd_port = statsd_port
+        self._statsd_prefix = statsd_prefix
+
         # Flag location to hold the peeked value of an argparse timeout value
         self._argv_timeout = False
 
@@ -337,11 +379,17 @@ class OpenStackConfig(object):
     def _load_yaml_json_file(self, filelist):
         for path in filelist:
             if os.path.exists(path):
-                with open(path, 'r') as f:
-                    if path.endswith('json'):
-                        return path, json.load(f)
-                    else:
-                        return path, yaml.safe_load(f)
+                try:
+                    with open(path, 'r') as f:
+                        if path.endswith('json'):
+                            return path, json.load(f)
+                        else:
+                            return path, yaml.safe_load(f)
+                except IOError as e:
+                    if e.errno == errno.EACCES:
+                        # Can't access file so let's continue to the next
+                        # file
+                        continue
         return (None, {})
 
     def _expand_region_name(self, region_name):
@@ -351,6 +399,13 @@ class OpenStackConfig(object):
         ret = []
         for region in regions:
             if isinstance(region, dict):
+                # i.e. must have name key, and only name,values keys
+                if 'name' not in region or \
+                   not {'name', 'values'} >= set(region):
+                    raise exceptions.ConfigException(
+                        'Invalid region entry at: %s' % region)
+                if 'values' not in region:
+                    region['values'] = {}
                 ret.append(copy.deepcopy(region))
             else:
                 ret.append(self._expand_region_name(region))
@@ -448,35 +503,43 @@ class OpenStackConfig(object):
         # Expand a profile if it exists. 'cloud' is an old confusing name
         # for this.
         profile_name = our_cloud.get('profile', our_cloud.get('cloud', None))
-        if profile_name and profile_name != self.envvar_key:
-            if 'cloud' in our_cloud:
-                warnings.warn(
-                    "{0} use the keyword 'cloud' to reference a known "
-                    "vendor profile. This has been deprecated in favor of the "
-                    "'profile' keyword.".format(self.config_filename))
-            vendor_filename, vendor_file = self._load_vendor_file()
-            if vendor_file and profile_name in vendor_file['public-clouds']:
-                _auth_update(cloud, vendor_file['public-clouds'][profile_name])
+        if not profile_name or profile_name == self.envvar_key:
+            return
+        if 'cloud' in our_cloud:
+            warnings.warn(
+                "{0} uses the keyword 'cloud' to reference a known "
+                "vendor profile. This has been deprecated in favor of the "
+                "'profile' keyword.".format(self.config_filename))
+
+        vendor_filename, vendor_file = self._load_vendor_file()
+        if (vendor_file and 'public-clouds' in vendor_file
+                and profile_name in vendor_file['public-clouds']):
+            _auth_update(cloud, vendor_file['public-clouds'][profile_name])
+        else:
+            profile_data = vendors.get_profile(profile_name)
+            if profile_data:
+                nested_profile = profile_data.pop('profile', None)
+                if nested_profile:
+                    nested_profile_data = vendors.get_profile(nested_profile)
+                    if nested_profile_data:
+                        profile_data = nested_profile_data
+                status = profile_data.pop('status', 'active')
+                message = profile_data.pop('message', '')
+                if status == 'deprecated':
+                    warnings.warn(
+                        "{profile_name} is deprecated: {message}".format(
+                            profile_name=profile_name, message=message))
+                elif status == 'shutdown':
+                    raise exceptions.ConfigException(
+                        "{profile_name} references a cloud that no longer"
+                        " exists: {message}".format(
+                            profile_name=profile_name, message=message))
+                _auth_update(cloud, profile_data)
             else:
-                profile_data = vendors.get_profile(profile_name)
-                if profile_data:
-                    status = profile_data.pop('status', 'active')
-                    message = profile_data.pop('message', '')
-                    if status == 'deprecated':
-                        warnings.warn(
-                            "{profile_name} is deprecated: {message}".format(
-                                profile_name=profile_name, message=message))
-                    elif status == 'shutdown':
-                        raise exceptions.ConfigException(
-                            "{profile_name} references a cloud that no longer"
-                            " exists: {message}".format(
-                                profile_name=profile_name, message=message))
-                    _auth_update(cloud, profile_data)
-                else:
-                    # Can't find the requested vendor config, go about business
-                    warnings.warn("Couldn't find the vendor profile '{0}', for"
-                                  " the cloud '{1}'".format(profile_name,
-                                                            name))
+                # Can't find the requested vendor config, go about business
+                warnings.warn("Couldn't find the vendor profile '{0}', for"
+                              " the cloud '{1}'".format(profile_name,
+                                                        name))
 
     def _project_scoped(self, cloud):
         return ('project_id' in cloud or 'project_name' in cloud
@@ -716,8 +779,8 @@ class OpenStackConfig(object):
             # legacy clients have un-prefixed api-version options
             parser.add_argument(
                 '--{service_key}-api-version'.format(
-                    service_key=service_key.replace('_', '-'),
-                    help=argparse_mod.SUPPRESS))
+                    service_key=service_key.replace('_', '-')),
+                help=argparse_mod.SUPPRESS)
             adapter.register_service_adapter_argparse_arguments(
                 parser, service_type=service_key)
 
@@ -919,9 +982,9 @@ class OpenStackConfig(object):
     def option_prompt(self, config, p_opt):
         """Prompt user for option that requires a value"""
         if (
-                getattr(p_opt, 'prompt', None) is not None and
-                p_opt.dest not in config['auth'] and
-                self._pw_callback is not None
+                getattr(p_opt, 'prompt', None) is not None
+                and p_opt.dest not in config['auth']
+                and self._pw_callback is not None
         ):
             config['auth'][p_opt.dest] = self._pw_callback(p_opt.prompt)
         return config
@@ -948,9 +1011,9 @@ class OpenStackConfig(object):
         """Perform the set of magic argument fixups"""
 
         # Infer token plugin if a token was given
-        if (('auth' in config and 'token' in config['auth']) or
-                ('auth_token' in config and config['auth_token']) or
-                ('token' in config and config['token'])):
+        if (('auth' in config and 'token' in config['auth'])
+                or ('auth_token' in config and config['auth_token'])
+                or ('token' in config and config['token'])):
             config.setdefault('token', config.pop('auth_token', None))
 
         # These backwards compat values are only set via argparse. If it's
@@ -1077,6 +1140,10 @@ class OpenStackConfig(object):
             cache_class=self._cache_class,
             cache_arguments=self._cache_arguments,
             password_callback=self._pw_callback,
+            statsd_host=self._statsd_host,
+            statsd_port=self._statsd_port,
+            statsd_prefix=self._statsd_prefix,
+            influxdb_config=self._influxdb_config,
         )
     # TODO(mordred) Backwards compat for OSC transition
     get_one_cloud = get_one
